/*********************************************************************

  Case.java	(C) Kenneth J. Turner, Mark A. Jennings	23/11/01 V1.3

  This class defines the case object used to encapsulate the right-of-way
  cases between two boats.

  KJT 14/08/01:

    o Minor tidy-up of code

    o Rule handling change to array of indexes; the list of rules is
      just indexes into the rules array; positive means the rule has not
      been cited before, = 0 means no rule, negative means the rule has
      been cited before

*********************************************************************/

import waba.ui.*;
import waba.fx.*;
import waba.sys.*;

public class Case {
  private int rules[] = new int[RuleBook.NUM_RULES];
  private Boat a;
  private Boat b;
  private Case next = null;
  private String connect = "has right of way over";
  private boolean addedTo;

  // basic constructor - 2 boats and the applicable rules

  public Case (Boat boat1, Boat boat2, int ruleList[]) {
    int i;

    a = boat1;
    b = boat2;
    for (i = 0; i < RuleBook.NUM_RULES; i++)
      rules[i] = ruleList[i];
    addedTo = false;
  }

  // overloaded constructor - optional extra text for the ruling relationship

  public Case (Boat boat1, Boat boat2, int ruleList[], String text) {
    this (boat1, boat2, ruleList);
    connect = text;
  }

  // add more applicable rules to an existing case object..

  public void addToCase (int extras[]) {
    int i = 0;
    int j = 0;

    if (!addedTo && extras != null) {
      while (rules[i] != 0)			// get to free rule place
        i++;
      while (extras[j] != 0)			// add extra rules
	rules[i++] = extras[j++];
      addedTo = true;
    }
  }

  // get the next case in the linked chain

  public Case getNext () {
    return (next);
  }

  // return the non-right-of-way boat

  public Boat getOther () {
    return (b);
  }

  // return the right-of-way boat

  public Boat getRight () {
    return (a);
  }

  // get the rule-book index at the given rule index

  public int getRule (int i) {
    return (rules[i]);
  }

  // set the next case in the linked chain

  public void setNext (Case c) {
    next = c;
  }

  // textual output describing the case

  public String stateCase () {
    Rule rule;
    String caseText;
    int i = 0;
    int ruleIndex;

    if (b != null) {
      caseText =
        "Boat " + a.getName () + "\n " + connect +
          "\nBoat " + b.getName () + "\n\n";
      while (i < RuleBook.NUM_RULES &&
               (ruleIndex = rules[i]) != 0) {
	if (i > 0)
	  caseText += "\n\n";
	if (ruleIndex > 0) {
	  caseText +=
	    "Rule " + RuleBook.getCode (ruleIndex) + ":\n";
	  caseText += RuleBook.getText (ruleIndex);
	}
	else
	  caseText +=
	    "Rule " + RuleBook.getCode (-ruleIndex) + " (see above)";
	i++;
      }
    }
    else
      caseText = "Error making case!";
    return (caseText);
  }

  // suppress (negate) the rule-book index at the
  // given rule index

  public void suppressRule (int i) {
    rules[i] = -rules[i];
  }
}
